"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander = require("commander");
const fs = require("fs-extra");
const path = require("path");
const index_1 = require("./index");
const query_parser_1 = require("./query-parser");
const authTokenEnvVar = "AUTH_TOKEN";
const oauth = process.env[authTokenEnvVar];
function parseQueryFile(filename) {
    return __awaiter(this, void 0, void 0, function* () {
        return query_parser_1.parseQuery(JSON.parse(yield fs.readFile(filename, "utf-8")));
    });
}
function main(argv) {
    return __awaiter(this, void 0, void 0, function* () {
        commander
            .option("-f, --file [filename]", "specify rules filename (default: rules.js)", "rules.js")
            .option("-d, --dry", "don't actually change anything", false)
            .option("-r, --rules [rules]", "specify a comma-delimited list of specific rules to run")
            .option("--ls", "displays a list of rule names and exits")
            .option("-q, --query [query]", "specify query filenames (comma-delimited, default: query.json)")
            .option("--single [ref]", "run on the specified issue or PR. Ref: owner/repo#id")
            .parse(process.argv);
        commander.usage("idembot --query open-issues.json");
        if (argv.length < 2) {
            commander.outputHelp();
            return 0;
        }
        const filename = commander.file;
        const queries = (commander.query && commander.query.split(",")) || [];
        console.log(queries);
        const dry = !!commander.dry;
        const single = commander.single || "";
        const cacheRoot = path.join(path.dirname(path.resolve(filename)), "cache");
        // Load the rules module
        let rulesMod;
        try {
            rulesMod = require(path.resolve(filename));
        }
        catch (e) {
            console.log(`Failed to load rules module ${filename}: ${e}`);
            return -1;
        }
        // Handle --ls mode and exit
        if (commander.ls) {
            console.log(`Rules in module ${filename}:`);
            for (const rule of Object.keys(rulesMod.rules)) {
                console.log(` * ${rule}`);
            }
            return 0;
        }
        const ruleNames = commander.rules || Object.keys(rulesMod.rules);
        // Parse queries
        let parsedOptions;
        if (single.length > 0) {
            if (queries.length > 0) {
                console.log('Specify only one of "queries" and "single"');
                return -1;
            }
            const match = /(\w+)\/(\w+)#(\w+)/.exec(single);
            if (!match) {
                console.log("Specify single in format: owner/name#id");
                return -1;
            }
            const [owner, name, id] = match.slice(1);
            parsedOptions = {
                cacheRoot, dry, ruleNames,
                kind: "single",
                single: { owner, name, id },
            };
        }
        else {
            const parsedQueries = [];
            if (queries.length === 0)
                queries.push("query.json");
            for (const q of queries || ["query.json"]) {
                try {
                    parsedQueries.push(yield parseQueryFile(q));
                }
                catch (e) {
                    console.log(`Error parsing query file ${q}: ${e}`);
                    return -1;
                }
            }
            parsedOptions = {
                cacheRoot, dry, ruleNames,
                kind: "queries",
                queries: parsedQueries,
            };
        }
        // Check for oauth token
        if (oauth === undefined) {
            console.error(`You must set the ${authTokenEnvVar} environment variable to a suitable OAuth token`);
            return -1;
        }
        const bot = index_1.default(rulesMod, parsedOptions, oauth);
        yield bot.runRules();
        return 0;
    });
}
main(process.argv).then((r) => process.exit(r));
//# sourceMappingURL=run.js.map